@@ -28,6 +28,7 @@ my $reIgnoreKeys    = qr/^$/;
 my $module          = __PACKAGE__;
 
 my @angularScopeVars;
+my @bool;
 my @cnodesKeys;
 my %cnodesRe;
 my @ignoreKeys;
@@ -288,6 +289,11 @@ $defaultAttr}
     my $sessionTypes = join( "', '", @sessionTypes );
     my $confConstants =
       "our \$hashParameters = qr/^" . $ra->as_string . "\$/;\n";
+    $ra = Regexp::Assemble->new;
+    foreach (@bool) {
+        $ra->add($_);
+    }
+    $confConstants .= "our \$boolKeys = qr/^" . $ra->as_string . "\$/;\n";
     open( F, ">", $self->confConstantsFile ) or die($!);
     print F <<EOF;
 # This file is generated by $module. Don't modify it by hand
@@ -340,6 +346,7 @@ our %EXPORT_TAGS = (
           NO
           \$hashParameters
           \@sessionTypes
+          \$boolKeys
           )
     ]
 );
@@ -562,6 +569,9 @@ sub scanTree {
                 if ( $prefix and !$jleaf->{get} ) {
                     $jleaf->{get} = $prefix . $jleaf->{title};
                 }
+                if ( $attr->{type} eq 'bool' ) {
+                    push @bool, $leaf;
+                }
             }
             push @$json, $jleaf;
         }
