@@ -3,7 +3,7 @@ package Lemonldap::NG::Manager::Conf::Parser;
 # This module is called either to parse a new configuration in JSON format (as
 # posted by the web interface) and test a new configuration object.
 #
-# The new object must be build with the following properties:
+# The new object must be built with the following properties:
 #  - refConf: the actual configuration
 #  - req    : the Lemonldap::NG::Common::PSGI::Request
 #  - tree   : the new configuration in JSON format
@@ -20,11 +20,13 @@ package Lemonldap::NG::Manager::Conf::Parser;
 
 use strict;
 use utf8;
+use Crypt::URandom;
 use Mouse;
+use JSON 'to_json';
 use Lemonldap::NG::Common::Conf::ReConstants;
 use Lemonldap::NG::Manager::Attributes;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.4';
 
 extends 'Lemonldap::NG::Common::Conf::Compact';
 
@@ -54,10 +56,10 @@ has message => (
         hdebug( "Message becomes " . $_[0]->{message} );
     }
 );
+has needConfirmation =>
+  ( is => 'rw', isa => 'ArrayRef', default => sub { return [] } );
 
 # Booleans
-has needConfirm =>
-  ( is => 'rw', isa => 'ArrayRef', default => sub { return [] } );
 has confChanged => (
     is      => 'rw',
     isa     => 'Bool',
@@ -68,7 +70,7 @@ has confChanged => (
 );
 
 # Properties required during build
-has refConf => ( is => 'ro', isa => 'HashRef', required => 1 );
+has refConf => ( is => 'ro', isa      => 'HashRef', required => 1 );
 has req     => ( is => 'ro', required => 1 );
 has newConf => ( is => 'rw', isa      => 'HashRef' );
 has tree    => ( is => 'rw', isa      => 'ArrayRef' );
@@ -79,6 +81,7 @@ sub hdebug {
         foreach my $d (@_) {
             if ( ref $d ) {
                 require Data::Dumper;
+                $Data::Dumper::Useperl = 1;
                 print STDERR Data::Dumper::Dumper($d);
             }
             else { print STDERR "$d\n" }
@@ -103,7 +106,7 @@ sub check {
     hdebug("  tests succeed");
     $self->compactConf( $self->newConf );
     unless ( $self->confChanged ) {
-        hdebug("  no changes detected");
+        hdebug("  no change detected");
         $self->message('__confNotChanged__');
         return 0;
     }
@@ -120,16 +123,17 @@ sub scanTree {
     $self->_scanNodes( $self->tree ) or return 0;
 
     # Set cfgNum to ref cfgNum (will be changed when saving), set other
-    # metadatas and set a value to the key if empty
+    # metadata and set a value to the key if empty
     $self->newConf->{cfgNum} = $self->req->params('cfgNum');
     $self->newConf->{cfgAuthor} =
-      $self->req->userData->{ $Lemonldap::NG::Handler::Main::tsv->{whatToTrace}
+      $self->req->userData->{ Lemonldap::NG::Handler::Main->tsv->{whatToTrace}
           || '_whatToTrace' } // "anonymous";
     $self->newConf->{cfgAuthorIP} = $self->req->address;
     $self->newConf->{cfgDate}     = time;
-    $self->newConf->{cfgVersion}  = $VERSION;
-    $self->newConf->{key} ||=
-      join( '', map { chr( int( rand(94) ) + 33 ) } ( 1 .. 16 ) );
+    $self->newConf->{cfgVersion}  = $Lemonldap::NG::Manager::VERSION;
+    $self->newConf->{key} ||= join( '',
+        map { chr( int( ord( Crypt::URandom::urandom(1) ) * 94 / 256 ) + 33 ) }
+          ( 1 .. 16 ) );
 
     return 1;
 }
@@ -156,7 +160,7 @@ sub _scanNodes {
         hdebug("Looking to $name");
 
         # subnode
-        my $subNodes     = $leaf->{nodes}      // $leaf->{_nodes};
+        my $subNodes     = $leaf->{nodes} // $leaf->{_nodes};
         my $subNodesCond = $leaf->{nodes_cond} // $leaf->{_nodes_cond};
 
         ##################################
@@ -304,7 +308,8 @@ sub _scanNodes {
             # SAML
             elsif ( $base =~ /^saml(?:S|ID)PMetaDataNodes$/ ) {
                 hdebug('SAML');
-                if ( defined $leaf->{data} and ref( $leaf->{data} ) eq 'ARRAY' )
+                if ( defined $leaf->{data}
+                    and ref( $leaf->{data} ) eq 'ARRAY' )
                 {
                     hdebug("  SAML data is an array, serializing");
                     $leaf->{data} = join ';', @{ $leaf->{data} };
@@ -317,6 +322,7 @@ sub _scanNodes {
                     }
                     elsif ($h) {
                         hdebug("  $target: opened node");
+                        $self->confChanged(1);
                         $self->set( $target, $key, $leaf->{title},
                             $leaf->{data} );
                     }
@@ -426,6 +432,74 @@ sub _scanNodes {
                 }
                 next;
             }
+
+            # CAS
+            elsif ( $base =~ /^cas(?:App|Srv)MetaDataNodes$/ ) {
+                my $optKey = $&;
+                hdebug('CAS');
+                if ( $target =~ /^cas(?:App|Srv)MetaDataOptions$/ ) {
+                    hdebug("  $target: looking for subnodes");
+                    $self->_scanNodes($subNodes);
+                    $self->set( $target, $key, $leaf->{title}, $leaf->{data} );
+                }
+                elsif ( $target =~ /^cas(?:App|Srv)MetaDataExportedVars$/ ) {
+                    hdebug("  $target");
+                    if ( $leaf->{cnodes} ) {
+                        hdebug('    unopened');
+                        $self->newConf->{$target}->{$key} =
+                          $self->refConf->{$target}->{$oldName} // {};
+                    }
+                    elsif ($h) {
+                        hdebug('    opened');
+                        $self->set( $target, $key, $leaf->{title},
+                            $leaf->{data} );
+                    }
+                    else {
+                        hdebug("  $target: looking for subnodes");
+                        $self->_scanNodes($subNodes);
+                    }
+                }
+                elsif ( $target =~ /^cas(?:Srv|App)MetaDataOptions/ ) {
+                    my $optKey = $&;
+                    hdebug "  $base sub key: $target";
+                    if ( $target eq 'casSrvMetaDataOptionsProxiedServices' ) {
+                        if ( $leaf->{cnodes} ) {
+                            hdebug('    unopened');
+                            $self->newConf->{$target}->{$key} =
+                              $self->refConf->{$target}->{$oldName} // {};
+                        }
+                        elsif ($h) {
+                            hdebug('    opened');
+                            $self->set( $target, $key, $leaf->{title},
+                                $leaf->{data} );
+                        }
+                        else {
+                            hdebug("  $target: looking for subnodes");
+                            $self->_scanNodes($subNodes);
+                        }
+                    }
+                    elsif ( $target =~
+                        /^(?:$casSrvMetaDataNodeKeys|$casAppMetaDataNodeKeys)/o
+                      )
+                    {
+                        $self->set(
+                            $optKey, [ $oldName, $key ],
+                            $target, $leaf->{data}
+                        );
+                    }
+                    else {
+                        push @{ $self->errors },
+                          { message => "Unknown CAS metadata option $target" };
+                        return 0;
+                    }
+                }
+                else {
+                    push @{ $self->errors },
+                      { message => "Unknown CAS option $target" };
+                    return 0;
+                }
+                next;
+            }
             else {
                 push @{ $self->errors },
                   { message => "Fatal: unknown special sub node $base" };
@@ -461,15 +535,25 @@ sub _scanNodes {
                         : {}
                     }
                   );
-                foreach my $cat (@listCatNew) {
-                    @listCatRef = grep { $_ ne $cat } @listCatRef;
-                }
-                if (@listCatRef) {
+
+                @listCatRef = map { $_ ? $_ : () } @listCatRef;
+                @listCatNew = map { $_ ? $_ : () } @listCatNew;
+                @listCatRef = sort @listCatRef;
+                @listCatNew = sort @listCatNew;
+                hdebug( '# @listCatRef : ', \@listCatRef );
+                hdebug( '# @listCatNew : ', \@listCatNew );
+
+                # Check for deleted
+                my @diff =
+                  grep !${ { map { $_, 1 } @listCatNew } }{$_}, @listCatRef;
+                if ( scalar @diff ) {
                     $self->confChanged(1);
-                    foreach my $cat (@listCatRef) {
-                        push @{ $self->changes },
-                          { key => $leaf->{id}, old => $cat };
-                    }
+                    push @{ $self->changes },
+                      {
+                        new => join( ', ', 'categoryList',      @listCatNew ),
+                        key => join( ', ', 'Deletes in cat(s)', @diff ),
+                        old => join( ', ', 'categoryList',      @listCatRef ),
+                      };
                 }
             }
             next;
@@ -512,8 +596,7 @@ sub _scanNodes {
                 $knownCat->{__id}++;
                 my $s = $knownCat->{$app} = sprintf '%04d-cat',
                   $knownCat->{__id};
-                $cn->{$s} =
-                  { catname => $leaf->{title}, type => 'category' };
+                $cn->{$s} = { catname => $leaf->{title}, type => 'category' };
                 unless ($cmp->{$app}
                     and $cmp->{$app}->{catname} eq $cn->{$s}->{catname} )
                 {
@@ -644,25 +727,6 @@ sub _scanNodes {
             next;
         }
 
-        # combModules: just to replace "over" key
-        elsif ( $name eq 'combModules' ) {
-            hdebug('combModules');
-            $self->newConf->{$name} = {};
-            foreach my $node ( @{ $leaf->{nodes} } ) {
-                my $tmp;
-                $tmp->{$_} = $node->{data}->{$_} foreach (qw(type for));
-                $tmp->{over} = {};
-                foreach ( @{ $node->{data}->{over} } ) {
-                    $tmp->{over}->{ $_->[0] } = $_->[1];
-                }
-                $self->newConf->{$name}->{ $node->{title} } = $tmp;
-            }
-
-            # TODO: check changes
-            $self->confChanged(1);
-            next;
-        }
-
         ####################
         # Other hash nodes #
         ####################
@@ -678,6 +742,26 @@ sub _scanNodes {
             }
             else {
                 hdebug('  opened');
+
+                # combModules: just to replace "over" key
+                if ( $name eq 'combModules' ) {
+                    hdebug('     combModules');
+                    $self->newConf->{$name} = {};
+                    foreach my $node ( @{ $leaf->{nodes} } ) {
+                        my $tmp;
+                        $tmp->{$_} = $node->{data}->{$_} foreach (qw(type for));
+                        $tmp->{over} = {};
+                        foreach ( @{ $node->{data}->{over} } ) {
+                            $tmp->{over}->{ $_->[0] } = $_->[1];
+                        }
+                        $self->newConf->{$name}->{ $node->{title} } = $tmp;
+                    }
+
+                    # TODO: check changes
+                    $self->confChanged(1);
+                    next;
+                }
+
                 $subNodes //= [];
                 my $count = 0;
                 my @old   = (
@@ -689,6 +773,14 @@ sub _scanNodes {
                 foreach my $n (@$subNodes) {
                     hdebug("  looking at $n subnode");
                     if ( ref $n->{data} and ref $n->{data} eq 'ARRAY' ) {
+
+                        # authChoiceModules
+                        if ( $name eq 'authChoiceModules' ) {
+                            hdebug('     combModules');
+                            $n->{data}->[5] ||= {};
+                            $n->{data}->[5] = to_json( $n->{data}->[5] );
+                        }
+
                         $n->{data} = join ';', @{ $n->{data} };
                     }
                     $self->newConf->{$name}->{ $n->{title} } = $n->{data};
@@ -752,7 +844,8 @@ sub _scanNodes {
                     @oldKeys = keys %{ $self->refConf->{$name}->{$host} };
                 }
                 foreach my $prm ( @{ $getHost->{h} } ) {
-                    $self->newConf->{$name}->{$host}->{ $prm->{k} } = $prm->{v};
+                    $self->newConf->{$name}->{$host}->{ $prm->{k} } =
+                      $prm->{v};
                     if (
                         !$change
                         and (
@@ -901,7 +994,7 @@ sub set {
 
 sub defaultValue {
     my ( $self, $target ) = @_;
-    hdebug("# defautValue($target)");
+    hdebug("# defaultValue($target)");
     die unless ($target);
     my $res = eval {
         &Lemonldap::NG::Manager::Attributes::attributes()->{$target}
@@ -931,6 +1024,12 @@ sub _unitTest {
     my $attrs = &Lemonldap::NG::Manager::Attributes::attributes();
     my $res   = 1;
     foreach my $key ( keys %$conf ) {
+        if (    $self->{skippedUnitTests}
+            and $self->{skippedUnitTests} =~ /\b$key\b/ )
+        {
+            $self->logger->debug("Ignore test for $key");
+            next;
+        }
         hdebug("Testing $key");
         my $attr = $attrs->{$key};
         my $type = $types->{ $attr->{type} };
@@ -971,15 +1070,14 @@ sub _unitTest {
                 or $attr->{type} =~ /Container$/ )
             {
                 my $keyMsg = $attr->{keyMsgFail} // $type->{keyMsgFail};
-                my $msg    = $attr->{msgFail}    // $type->{msgFail};
+                my $msg    = $attr->{msgFail} // $type->{msgFail};
                 $res = 0
                   unless (
-                    $self->_execTest(
-                        {
-                            keyTest => $attr->{keyTest} // $type->{keyTest},
+                    $self->_execTest( {
+                            keyTest    => $attr->{keyTest} // $type->{keyTest},
                             keyMsgFail => $attr->{keyMsgFail}
                               // $type->{keyMsgFail},
-                            test    => $attr->{test}    // $type->{test},
+                            test    => $attr->{test} // $type->{test},
                             msgFail => $attr->{msgFail} // $type->{msgFail},
                         },
                         $conf->{$key},
@@ -1065,12 +1163,18 @@ sub _globalTest {
     my $result = 1;
     my $tests  = &Lemonldap::NG::Manager::Conf::Tests::tests( $self->newConf );
     foreach my $name ( keys %$tests ) {
+        if (    $self->{skippedGlobalTests}
+            and $self->{skippedGlobalTests} =~ /\b$name\b/ )
+        {
+            $self->logger->debug("Ignore test for $name");
+            next;
+        }
         my $sub = $tests->{$name};
         my ( $res, $msg );
         eval {
             ( $res, $msg ) = $sub->();
             if ( $res == -1 ) {
-                push @{ $self->needConfirm }, { message => $msg };
+                push @{ $self->needConfirmation }, { message => $msg };
             }
             elsif ($res) {
                 if ($msg) {
@@ -1141,10 +1245,10 @@ L<Lemonldap::NG::Manager>, L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 If you want to report a bug concerning configuration upload, please change
-HIGHDEBUG constant value to produce more logs. Then post them in the Jira
+HIGHDEBUG constant value to produce more logs. Then post them in the Gitlab
 ticket.
 
 =head1 DOWNLOAD
