@@ -4,6 +4,7 @@ use 5.10.0;
 use utf8;
 use Mouse;
 use JSON qw(from_json to_json);
+use POSIX qw(strftime);
 
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::PSGI::Constants;
@@ -14,7 +15,7 @@ use feature 'state';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 has notifAccess => ( is => 'rw' );
 
@@ -108,8 +109,7 @@ sub setNotifAccess {
     unless (
         eval {
             $self->notifAccess(
-                $type->new(
-                    {
+                $type->new( {
                         %{ $self->{notificationStorageOptions} },
                         p    => $self,
                         conf => $self
@@ -262,11 +262,32 @@ sub newNotification {
         return $self->sendError( $req, undef, 200 );
     }
 
-    foreach my $r (qw(uid date reference xml)) {
+    foreach my $r (qw(uid reference xml)) {
         return $self->sendError( $req, "Missing $r", 200 )
           unless ( $json->{$r} );
     }
 
+    # Set default date value
+    my $dDate = strftime( "%Y-%m-%d", localtime() );
+    if ( $json->{date} ) {
+        $self->logger->debug(
+"Posted data : uid = $json->{uid} - Ref = $json->{reference} - Date = $json->{date}"
+        );
+    }
+    else {
+        $self->logger->debug(
+"Posted data : uid = $json->{uid} - Ref = $json->{reference} - Date = ???"
+        );
+        $json->{date} = $dDate;
+    }
+
+    # Check if posted date > today
+    unless ( $json->{date} ge $dDate ) {
+        $self->logger->debug("Posted Date < today !!! ");
+        $json->{date} = $dDate;
+    }
+    $self->logger->debug("Notification Date = $json->{date}");
+
     unless ( $json->{date} =~ /^\d{4}-\d{2}-\d{2}$/ ) {
         return $self->sendError( $req, "Malformed date", 200 );
     }
@@ -286,7 +307,7 @@ sub newNotification {
     }
     else {
         eval {
-            my $tmp = from_json( $json->{xml} );
+            my $tmp = from_json( $json->{xml}, { allow_nonref => 1 } );
             $json->{$_} = $tmp->{$_} foreach ( keys %$tmp );
             delete $json->{xml};
         };
