@@ -17,6 +17,11 @@ function templates(tpl,key) {
             "id" : tpl+"s/"+key+"/"+"casAppMetaDataOptionsService",
             "title" : "casAppMetaDataOptionsService"
          },
+         {
+            "get" : tpl+"s/"+key+"/"+"casAppMetaDataOptionsUserAttribute",
+            "id" : tpl+"s/"+key+"/"+"casAppMetaDataOptionsUserAttribute",
+            "title" : "casAppMetaDataOptionsUserAttribute"
+         },
          {
             "get" : tpl+"s/"+key+"/"+"casAppMetaDataOptionsRule",
             "id" : tpl+"s/"+key+"/"+"casAppMetaDataOptionsRule",
@@ -405,6 +410,20 @@ function templates(tpl,key) {
                   "id" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsClientSecret",
                   "title" : "oidcRPMetaDataOptionsClientSecret",
                   "type" : "password"
+               },
+               {
+                  "default" : 0,
+                  "get" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsPublic",
+                  "id" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsPublic",
+                  "title" : "oidcRPMetaDataOptionsPublic",
+                  "type" : "bool"
+               },
+               {
+                  "default" : 0,
+                  "get" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsRequirePKCE",
+                  "id" : tpl+"s/"+key+"/"+"oidcRPMetaDataOptionsRequirePKCE",
+                  "title" : "oidcRPMetaDataOptionsRequirePKCE",
+                  "type" : "bool"
                }
             ],
             "id" : "oidcRPMetaDataOptionsAuthentication",
@@ -1139,22 +1158,10 @@ function templates(tpl,key) {
             "get" : tpl+"s/"+key+"/"+"vhostType",
             "id" : tpl+"s/"+key+"/"+"vhostType",
             "select" : [
-               {
-                  "k" : "Main",
-                  "v" : "Main"
-               },
-               {
-                  "k" : "Zimbra",
-                  "v" : "ZimbraPreAuth"
-               },
                {
                   "k" : "AuthBasic",
                   "v" : "AuthBasic"
                },
-               {
-                  "k" : "SecureToken",
-                  "v" : "SecureToken"
-               },
                {
                   "k" : "CDA",
                   "v" : "CDA"
@@ -1167,9 +1174,25 @@ function templates(tpl,key) {
                   "k" : "DevOpsST",
                   "v" : "DevOpsST"
                },
+               {
+                  "k" : "Main",
+                  "v" : "Main"
+               },
+               {
+                  "k" : "OAuth2",
+                  "v" : "OAuth2"
+               },
+               {
+                  "k" : "SecureToken",
+                  "v" : "SecureToken"
+               },
                {
                   "k" : "ServiceToken",
                   "v" : "ServiceToken"
+               },
+               {
+                  "k" : "Zimbra",
+                  "v" : "ZimbraPreAuth"
                }
             ],
             "title" : "vhostType",
